@@ -67,7 +67,7 @@ def _should_run(cycle: dict):
     def _get_measurement_content(term_id: str):
         return most_relevant_measurement_value(measurements, term_id, end_date)
 
-    histosol = _get_measurement_content('histosol') or 0
+    histosol = _get_measurement_content('histosol')
     eco_climate_zone = _get_measurement_content('ecoClimateZone')
     organic_soil_factor = _get_CO2_factor(eco_climate_zone, site_type) if eco_climate_zone else 0
     land_occupation = land_occupation_per_ha(MODEL, TERM_ID, cycle)
@@ -77,7 +77,7 @@ def _should_run(cycle: dict):
                     land_occupation=land_occupation,
                     histosol=histosol)
 
-    should_run = all([organic_soil_factor, land_occupation])
+    should_run = all([organic_soil_factor, land_occupation, histosol is not None])
     logShouldRun(cycle, MODEL, TERM_ID, should_run, methodTier=TIER)
     return should_run, histosol, organic_soil_factor, land_occupation
 
