@@ -13,16 +13,15 @@ a simplified formula is used: total nitrogen content of the fish * 3.31.
 See [Poore & Nemecek (2018)](https://science.sciencemag.org/content/360/6392/987) for further details.
 """
 from hestia_earth.schema import TermTermType
-from hestia_earth.utils.model import find_primary_product, find_term_match
+from hestia_earth.utils.model import find_primary_product
 from hestia_earth.utils.tools import list_sum
 
 from hestia_earth.models.log import logRequirements, logShouldRun
-from hestia_earth.models.utils.term import get_lookup_value
 from hestia_earth.models.utils.property import _get_nitrogen_content
 from hestia_earth.models.utils.input import get_feed_inputs
 from hestia_earth.models.utils.product import _new_product, get_animal_produced_nitrogen
 from hestia_earth.models.utils.blank_node import convert_to_nitrogen
-from hestia_earth.models.utils.term import get_excreta_N_terms
+from .utils import get_excreta_product
 from . import MODEL
 
 REQUIREMENTS = {
@@ -74,48 +73,44 @@ RETURNS = {
 }
 LOOKUPS = {
     "crop-property": ["nitrogenContent", "crudeProteinContent"],
-    "animalProduct": "excretaKgNTermId",
-    "liveAnimal": "excretaKgNTermId",
-    "liveAquaticSpecies": "excretaKgNTermId"
+    "animalProduct": ["excretaKgNTermId", "allowedExcretaKgNTermIds"],
+    "liveAnimal": ["excretaKgNTermId", "allowedExcretaKgNTermIds"],
+    "liveAquaticSpecies": ["excretaKgNTermId", "allowedExcretaKgNTermIds"]
 }
 MODEL_KEY = 'excretaKgN'
-MODEL_LOG = '/'.join([MODEL, MODEL_KEY])
 
 
-def _product(value: float, excreta: str):
-    product = _new_product(excreta, value, MODEL)
-    return product
+def _product(excreta_product: str, value: float):
+    product = _new_product(excreta_product.get('term', {}).get('@id'), value, MODEL)
+    return excreta_product | product
 
 
-def _run(term_id: str, mass_balance_items: list, alternate_items: list):
+def _run(excreta_product: dict, mass_balance_items: list, alternate_items: list):
     inputs_n, products_n = mass_balance_items
     product_value, nitrogen_content = alternate_items
     value = inputs_n - products_n if all(mass_balance_items) else 3.31 * product_value * nitrogen_content / 100
-    return [_product(value, term_id)] if value > 0 else []
+    return [_product(excreta_product, value)] if value > 0 else []
 
 
-def _no_excreta_term(products: list):
-    term_ids = get_excreta_N_terms()
-    return all([not find_term_match(products, term) for term in term_ids])
-
-
-def _get_excreta_n_term(product: dict):
-    term = product.get('term', {})
-    return get_lookup_value(term, LOOKUPS.get(term.get('termType', TermTermType.ANIMALPRODUCT.value)), model=MODEL)
+def _get_excreta_product(cycle: dict, primary_product: dict):
+    term = primary_product.get('term', {})
+    return get_excreta_product(cycle, term, 'excretaKgNTermId', 'allowedExcretaKgNTermIds')
 
 
 def _should_run(cycle: dict):
     primary_prod = find_primary_product(cycle) or {}
-    excreta_term_id = _get_excreta_n_term(primary_prod)
+    excreta_product = _get_excreta_product(cycle, primary_prod)
+    excreta_term_id = excreta_product.get('term', {}).get('@id')
+    should_add_product = all([not excreta_product.get('value', [])])
+
     dc = cycle.get('completeness', {})
     is_animalFeed_complete = dc.get('animalFeed', False)
     is_product_complete = dc.get('product', False)
 
     inputs_feed = get_feed_inputs(cycle)
-    inputs_n = convert_to_nitrogen(cycle, MODEL_LOG, excreta_term_id, inputs_feed)
+    inputs_n = convert_to_nitrogen(cycle, MODEL, excreta_term_id, inputs_feed, model_key=MODEL_KEY)
 
-    products_n = get_animal_produced_nitrogen(MODEL_LOG, cycle.get('products', []))
-    should_add_product = _no_excreta_term(cycle.get('products', []))
+    products_n = get_animal_produced_nitrogen(MODEL, cycle.get('products', []))
 
     # we can still run the model for `liveAquaticSpecies`
     is_liveAquaticSpecies = primary_prod.get('term', {}).get('termType') == TermTermType.LIVEAQUATICSPECIES.value
@@ -123,13 +118,13 @@ def _should_run(cycle: dict):
     nitrogen_content = _get_nitrogen_content(primary_prod)
 
     if is_liveAquaticSpecies:
-        logRequirements(cycle, model=MODEL_LOG, term=excreta_term_id,
+        logRequirements(cycle, model=MODEL, term=excreta_term_id, model_key=MODEL_KEY,
                         is_liveAquaticSpecies=is_liveAquaticSpecies,
                         product_value=product_value,
                         nitrogen_content=nitrogen_content)
 
     else:
-        logRequirements(cycle, model=MODEL_LOG, term=excreta_term_id,
+        logRequirements(cycle, model=MODEL, term=excreta_term_id, model_key=MODEL_KEY,
                         is_animalFeed_complete=is_animalFeed_complete,
                         is_product_complete=is_product_complete,
                         inputs_n=inputs_n,
@@ -148,11 +143,11 @@ def _should_run(cycle: dict):
     ])
     # only log if the excreta term does not exist to avoid showing failure when it already exists
     if should_add_product:
-        logShouldRun(cycle, MODEL_LOG, excreta_term_id, should_run)
-    logShouldRun(cycle, MODEL_LOG, None, should_run)
-    return should_run, excreta_term_id, mass_balance_items, alternate_items
+        logShouldRun(cycle, MODEL, excreta_term_id, should_run, model_key=MODEL_KEY)
+    logShouldRun(cycle, MODEL, None, should_run)
+    return should_run, excreta_product, mass_balance_items, alternate_items
 
 
 def run(cycle: dict):
-    should_run, term_id, mass_balance_items, alternate_items = _should_run(cycle)
-    return _run(term_id, mass_balance_items, alternate_items) if should_run else []
+    should_run, excreta_product, mass_balance_items, alternate_items = _should_run(cycle)
+    return _run(excreta_product, mass_balance_items, alternate_items) if should_run else []
