@@ -20,7 +20,8 @@ def test_run_with_dryMatter(*args):
     assert result == expected
 
 
-def test_run_with_min_max():
+@patch('hestia_earth.models.utils.feedipedia._new_property', side_effect=fake_new_property)
+def test_run_with_min_max(*args):
     with open(f"{fixtures_folder}/with-min-max/cycle.jsonld", encoding='utf-8') as f:
         cycle = json.load(f)
 
