@@ -11,11 +11,17 @@ fixtures_folder = f"{fixtures_path}/{MODEL}/{TERM_ID}"
 @patch(f"{class_path}.should_download", return_value=True)
 @patch(f"{class_path}.has_geospatial_data")
 def test_should_run(mock_has_geospatial_data, *args):
-    mock_has_geospatial_data.return_value = True
-    assert _should_run({}) is True
+    site = {}
 
     mock_has_geospatial_data.return_value = False
-    assert not _should_run({})
+    assert not _should_run(site)
+
+    mock_has_geospatial_data.return_value = True
+    assert _should_run(site) is True
+
+    # with other texture measurements => not run
+    site['measurements'] = [{'term': {'@id': 'clayContent'}}]
+    assert not _should_run(site)
 
 
 @patch(f"{class_path}.get_source", return_value={})
