@@ -9,11 +9,9 @@ fixtures_folder = f"{fixtures_path}/{MODEL}/excretaKgN"
 
 
 @patch(f"{class_path}._get_nitrogen_content", return_value=10)
-@patch(f"{class_path}.get_excreta_N_terms", return_value=[])
 @patch(f"{class_path}.convert_to_nitrogen", return_value=0)
 @patch(f"{class_path}.get_animal_produced_nitrogen", return_value=0)
-@patch(f"{class_path}._get_excreta_n_term", return_value='excretaKgN')
-def test_should_run(mock_get_excreta_term, mock_animal_produced, mock_get_feed, *args):
+def test_should_run(mock_animal_produced, mock_get_feed, *args):
     cycle = {}
     should_run, *args = _should_run(cycle)
     assert not should_run
@@ -30,7 +28,7 @@ def test_should_run(mock_get_excreta_term, mock_animal_produced, mock_get_feed,
     should_run, *args = _should_run(cycle)
     assert not should_run
 
-    # completeness True => run
+    # completeness True => no run
     cycle = {
         'completeness': {
             'animalFeed': True,
@@ -40,26 +38,28 @@ def test_should_run(mock_get_excreta_term, mock_animal_produced, mock_get_feed,
     mock_get_feed.return_value = 5
     mock_animal_produced.return_value = 5
     should_run, *args = _should_run(cycle)
-    assert should_run is True
-
-    # no excreta term => no run
-    mock_get_excreta_term.return_value = None
-    should_run, *args = _should_run(cycle)
     assert not should_run
 
-    # excreta already present => no run
+    # excreta already present with no value => run
     cycle['products'] = [
         {
             'term': {
-                '@id': 'excretaKgN'
+                '@id': 'broilerChicken',
+                'termType': 'liveAnimal'
+            }
+        },
+        {
+            'term': {
+                '@id': 'excretaKgN',
+                'termType': 'excreta'
             }
         }
     ]
     should_run, *args = _should_run(cycle)
-    assert not should_run
+    assert should_run is True
 
 
-@patch(f"{class_path}.get_excreta_N_terms", return_value=[])
+@patch("hestia_earth.models.utils.property.download_hestia", return_value={'units': '%'})
 @patch(f"{class_path}._new_product", side_effect=fake_new_product)
 def test_run(*args):
     with open(f"{fixtures_folder}/cycle.jsonld", encoding='utf-8') as f:
@@ -72,7 +72,7 @@ def test_run(*args):
     assert value == expected
 
 
-@patch(f"{class_path}.get_excreta_N_terms", return_value=[])
+@patch("hestia_earth.models.utils.property.download_hestia", return_value={'units': '%'})
 @patch(f"{class_path}._new_product", side_effect=fake_new_product)
 def test_run_with_liveweight(*args):
     with open(f"{fixtures_folder}/with-liveweight/cycle.jsonld", encoding='utf-8') as f:
@@ -85,7 +85,7 @@ def test_run_with_liveweight(*args):
     assert value == expected
 
 
-@patch(f"{class_path}.get_excreta_N_terms", return_value=[])
+@patch("hestia_earth.models.utils.property.download_hestia", return_value={'units': '%'})
 @patch(f"{class_path}._new_product", side_effect=fake_new_product)
 def test_run_with_carcass(*args):
     with open(f"{fixtures_folder}/with-carcass/cycle.jsonld", encoding='utf-8') as f:
@@ -98,7 +98,7 @@ def test_run_with_carcass(*args):
     assert value == expected
 
 
-@patch(f"{class_path}.get_excreta_N_terms", return_value=[])
+@patch("hestia_earth.models.utils.property.download_hestia", return_value={'units': '%'})
 @patch(f"{class_path}._new_product", side_effect=fake_new_product)
 def test_run_with_head(*args):
     with open(f"{fixtures_folder}/with-head/cycle.jsonld", encoding='utf-8') as f:
@@ -111,7 +111,7 @@ def test_run_with_head(*args):
     assert value == expected
 
 
-@patch(f"{class_path}.get_excreta_N_terms", return_value=[])
+@patch("hestia_earth.models.utils.property.download_hestia", return_value={'units': '%'})
 @patch(f"{class_path}._new_product", side_effect=fake_new_product)
 def test_run_with_liveAquaticSpecies(*args):
     with open(f"{fixtures_folder}/with-liveAquaticSpecies/cycle.jsonld", encoding='utf-8') as f:
