@@ -21,7 +21,7 @@ def test_run(*args):
 
 
 @patch(f"{class_path}._new_practice", side_effect=fake_new_practice)
-def test_run_no_orchard_crop(*args):
+def test_run_no_plantation(*args):
     with open(f"{fixtures_folder}/no-orchard-crop/cycle.jsonld", encoding='utf-8') as f:
         cycle = json.load(f)
 
